from meta import util


class PlayerCharacter(object):
    def __init__(self):
        # Containers
        self.player_name = ""
        self.xp = 0
        self.level = 2
        self.gp = 0
        self.weapon = ""
        self.shield = ""
        self.armor = ""
        self.helm = ""
        self.inventory = []  # List of objects
        self.spell_book = []

        # Attributes
        self.attribute_points = 0
        self.strength = 10
        self.dexterity = 5
        self.vitality = 6
        self.wisdom = 10

        # Stats
        self.attack = 0
        self.defence = 0
        self.max_health = 0
        self.current_health = 0
        self.max_ability_power = 0
        self.ability_power = 0
        self.chance_to_hit = 0.0

    def equip(self):
        """
        Lists inventory of the equipment type
        Produces a menu to choose which item to equip
        Returns current equipment of chosen type back to inventory, if there is one equipped
        Equips selected equipment to the equipment slot of that type
        :param equipment:
        :return:
        """
        print("\nPlease select the item to equip:")
        # A list of inventory with the equipment type
        equipment = []
        for item in self.inventory:
            if item.item_type:
                equipment.append(item)
        for i, e in enumerate(equipment):
            print(i + 1, ":", e)
        ev = util.EventHandler()
        # Plays animation
        ev.wait_for_user()

        # Captures the keypress from the user
        player_input = ev.key_pressed
        player_input = int(player_input) - 1
        util.clear()

        # Indicates the item chosen
        chosen_equipment = equipment[player_input]

        # Verify chosen item's pre-reqs are met
        if (
            chosen_equipment.vitality_required <= self.vitality and
            chosen_equipment.strength_required <= self.strength and
            chosen_equipment.dexterity_required <= self.dexterity and
            chosen_equipment.wisdom_required <= self.wisdom
           ):

            # Return equipped item, equip selected item, remove selected item from inventory
            if chosen_equipment.item_type == "weapon":
                if self.weapon:
                    self.inventory.append(self.weapon)
                self.weapon = chosen_equipment
                self.inventory.remove(chosen_equipment)

            if chosen_equipment.item_type == "shield":
                if self.shield:
                        self.inventory.append(self.shield)
                self.shield = chosen_equipment
                self.inventory.remove(chosen_equipment)

            if chosen_equipment.item_type == "armor":
                if self.armor:
                    self.inventory.append(self.armor)
                self.armor = chosen_equipment
                self.inventory.remove(chosen_equipment)

            if chosen_equipment.item_type == "helm":
                if self.helm:
                    self.inventory.append(self.helm)
                self.helm = chosen_equipment
                self.inventory.remove(chosen_equipment)

                # Change stats
                self.equip_stats()
        else:
            print("You are not skilled enough to equip that item")

    def equip_stats(self):
        if self.weapon:
            self.max_health += self.weapon.health_bonus
            self.attack += self.weapon.attack_bonus
            self.max_ability_power += self.weapon.ability_power_bonus
            self.defence += self.weapon.defence_bonus
        if self.armor:
            self.max_health += self.armor.health_bonus
            self.attack += self.armor.attack_bonus
            self.max_ability_power += self.armor.ability_power_bonus
            self.defence += self.armor.defence_bonus
        if self.shield:
            self.max_health += self.shield.health_bonus
            self.attack += self.shield.attack_bonus
            self.max_ability_power += self.shield.ability_power_bonus
            self.defence += self.shield.defence_bonus
        if self.helm:
            self.max_health += self.helm.health_bonus
            self.attack += self.helm.attack_bonus
            self.max_ability_power += self.helm.ability_power_bonus
            self.defence += self.helm.defence_bonus

    def leveler(self, xp):
        # if xp reaches a certain number in the level_up list, the player levels up.
        # the player gets to choose which attribute to increase
        level_up = [1000, 2000, 3000, 5000, 8000, 13000, 21000, 34000, 55000]
        level_bonuses = [1, 1.2, 1.4, 1.6, 1.8, 2.0, 2.2, 2.4, 2.6, 2.8]
        self.xp += xp
        if self.xp >= level_up[self.level - 1]:
            self.level += 1
            self.attribute_points += 1
            print("\nLevel up! \n" + self.player_name + " is now level " + str(self.level) + "!\n")
            print(self.player_name + " has " + str(self.attribute_points) + " attribute points to spend.")
            print("Level Bonus is a " + str(level_bonuses[self.level - 1]) + " modifier on attack and defence\n")
            self.current_health = self.max_health
            self.ability_power = self.max_ability_power

            print("Would you like to use your " + str(self.attribute_points) + " attribute points now?")
            player_choice = input("\n1: Yes\n2: No\n")
            if player_choice == "1":
                self.add_atribs()
            else:
                print("Ok. Saving those for later eh? Hmmmmm.")

        self.attack = int((self.strength + (self.dexterity / 2) / 2) * level_bonuses[self.level - 1])
        self.defence = int((self.vitality + (self.strength / 2) / 2) * level_bonuses[self.level - 1])
        self.max_health = int((self.vitality * 10) * (level_bonuses[self.level - 1] * 2))
        self.max_ability_power = int((self.wisdom + (self.vitality / 2) / 2) * level_bonuses[self.level - 1])
        self.equip_stats()

    def print_stats(self):
        print("Player name: ", self.player_name,"\n",
              "XP:          ", self.xp, "\n",
              "Level:       ", self.level, "\n",
              "Weapon:      ", self.weapon, "\n",
              "Shield:      ", self.shield, "\n",
              "Armor:       ", self.armor, "\n",
              "Helm:        ", self.helm, "\n",
              "Inventory:   ", self.inventory, "\n",
              "AP:          ", self.ability_power, "\n",
              "Strength:    ", self.strength, "\n",
              "Dexterity:   ", self.dexterity, "\n",
              "Vitality:    ", self.vitality, "\n",
              "Wisdom:      ", self.wisdom, "\n",
              "Attack:      ", self.attack, "\n",
              "Defence:     ", self.defence, "\n"
              )

    def add_atribs(self):
        while self.attribute_points > 0:
            print("You have " + str(self.attribute_points) + " to spend.")
            print("""
    Select an option:
    1: Strength
    2: Dexterity
    3: Vitality
    4: Wisdom
    5: Cancel\n""")
            player_choice = input(":")
            if player_choice == "1":
                self.strength += 1
                self.attribute_points -= 1
            if player_choice == "2":
                self.dexterity += 1
                self.attribute_points -= 1
            if player_choice == "3":
                self.vitality += 1
                self.attribute_points -= 1
            if player_choice == "4":
                self.wisdom += 1
                self.attribute_points -= 1
            if player_choice == "5":
                print("Exited Attribute Assignment")
                break
